import React from 'react';
import { useState } from 'react';

import * as d3 from 'd3';
import { useEffect } from 'react';
import { csv } from 'd3';
import { scaleSequential } from 'd3-scale';
import { useData } from './useData.js';
import { Text } from './Text.js';
import { extent, max, min } from 'd3-array';
////////////////////////////////////////////
const margin = {
  top: 100,
  right: 150,
  bottom: 200,
  left: 100,
};
const width = 1000;
const height = 1000;
const innerHeight =
  height - margin.top - margin.bottom;
const innerWidth =
  width - margin.left - margin.right;
/////////////////////////////////////////////////////

///////////////////////////////////////////////////
// Data function
///////////////////////////////////////////////////

const BarChart = ({data}) => {
  const [opacityVar, setopacityVar] = useState(
    0.9
  );
  const [
    strokeWidthVar,
    setstrokeWidthVar,
  ] = useState(2);
  const [index, setIndex] = useState(0);
 
  const [
    display_text_mouseover,
    setdisplay_text_mouseover,
  ] = useState('');

  const xValue = (d) =>
    d['Chargesheet-Rate-2021'];
  const yValue = (d) =>
    d['Rate-of-Total-Crime-against-Women-2021'];

  // const xValue = (d: any) => d["Rate-of-Total-Crime-against-Women-2021"];
  // const yValue = (d: any) => d["Chargesheet-Rate-2021"];
  const zValue = (d) => d['States/UT'];
  const xAxisLabel = 'Chargesheet rate';
  const yAxisLabel = 'Crime rate';
  const xScale = d3
    .scaleLinear()
    .domain([0, d3.max(data, xValue)])
    .range([0, innerWidth - 50])
    .nice();
  const yScale = d3
    .scaleLinear()
    .domain([d3.max(data, yValue), 0])
    .range([0, innerHeight])
    .nice();

  ///////////////////////////////////////////////////////////////////////////
  // const colorScale = d3
  //   .scaleOrdinal()
  //   .domain(data.map(zValue))
  //   .range(d3.schemeCategory10);

  const colorScale = d3
    .scaleOrdinal()
    .domain(data.map(zValue))
    .range(d3.schemeDark2);

  //////////////////////////////////////////////////////////////////////////
  // Axis//
  const yAxis = d3.axisLeft(yScale);
  const yAxisG = d3.select('g');
  yAxisG.call(yAxis);
  //////////////////////////////////////////////////////////////////////////
  const xAxis = d3.axisBottom(xScale);
  const xAxisG = d3
    .select('g')
    .append('g')
    .call(xAxis);
  xAxisG.attr(
    'transform',
    `translate(0,${
      innerHeight + margin.top - margin.bottom / 3
    })`
  );
  ////////////////////////////////////////////////////////////////////////
  //
  //////////////////////////////////////////////////////////////////////
  return (
    <>
      <svg width={width} height={height}>
        <text
          x={300}
          y={50}
          fontSize="22"
          fontWeight={'bold'}
          opacity={0.8}
        >
          {'Chartsheet Rate vs Crime Rate (2021)'}
        </text>
        <text
          x={innerWidth / 2}
          y={innerHeight + 100}
          fontSize="16"
          fontWeight={'bold'}
        >
          {xAxisLabel}
        </text>
        <text
          transform="rotate(-90) translate(-500,-250)"
          x={50}
          y={innerHeight / 2 - 50}
          fontSize="16"
          fontWeight={'bold'}
        >
          {' '}
          {yAxisLabel}
        </text>
        <g
          transform={`translate(${margin.left},${margin.top}) scale(0.9)`}
        >
          <>
            {data.map((d, i) => {
              //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
              return (
                <>
                  <g>
                    <circle
                      key={Math.random() + 10}
                      r={18}
                      cx={xScale(xValue(d))}
                      cy={yScale(yValue(d))}
                      padding={'50em' }
                      dy={'0.3em'}
                      fill={colorScale(zValue(d))}
                      // fill="steelblue"
                      stroke={'black'}
                      onClick={() => {
                        setIndex(i);
                        setopacityVar(1);
                        setstrokeWidthVar(8);
                      }}
                      onMouseOver={() => {
                        setIndex(i);
                        setopacityVar(1);
                        setstrokeWidthVar(6);
                        // setdisplay_text_mouseover(d["state"]);
                      }}
                      // strokeOpacity={i == index ? 1 : 0}
                      opacity={
                        i == index
                          ? opacityVar
                          : 0.5
                      }
                      strokeWidth={strokeWidthVar}
                    />

                    <Text
                      style={{
                        fontSize: '18px',
                        fill: 'black',
                        opacity: 1,
                        fontWeight: 'bold',
                      }}
                      key={
                        Math.random() +
                        Math.random() * 300
                      }
                      x={xScale(xValue(d)) - 200}
                      y={yScale(yValue(d)) + 35}
                      text={
                        i === index
                          ? d['States/UT'] +
                            ' ' +
                            '( ' +
                            'chargesheet rate: ' +
                            +d[
                              'Chargesheet-Rate-2021'
                            ] +
                            ' , ' +
                            'crime rate: ' +
                            +d[
                              'Rate-of-Total-Crime-against-Women-2021'
                            ] +
                            ' )'
                          : null
                      }
                    />
                  </g>
                  <g transform="translate(30)">
                    <rect
                      key={Math.random() + 10}
                      x={innerWidth}
                      y={i * 20}
                      width={20}
                      height={10}
                      dy={'0.3em'}
                      fill={colorScale(zValue(d))}
                      // fill="steelblue"
                      stroke={'black'}
                      onClick={() => {
                        setIndex(i);
                        setopacityVar(1);
                        setstrokeWidthVar(1);
                      }}
                      onMouseOver={() => {
                        setIndex(i);
                        setopacityVar(1);
                        setstrokeWidthVar(1);
                        // setdisplay_text_mouseover(d["state"]);
                      }}
                      // strokeOpacity={i == index ? 1 : 0}
                      opacity={
                        i == index
                          ? opacityVar
                          : 0.5
                      }
                      strokeWidth={strokeWidthVar}
                    />
                    <g transform="translate(0,4)">
                      <Text
                        style={{
                          fontSize: '16px',
                          fill: 'black',
                          opacity: 0.5,
                          fontWeight: 'lighter',
                        }}
                        key={
                          Math.random() +
                          Math.random() * 300
                        }
                        x={innerWidth - 50}
                        y={i * 20}
                        text={d['SL']}
                      />
                      <Text
                        style={{
                          fontSize: '16px',
                          fill: 'black',
                          opacity: 1,
                          fontWeight: 'normal',
                        }}
                        key={
                          Math.random() +
                          Math.random() * 300
                        }
                        x={innerWidth + 10}
                        y={i * 20}
                        text={d['States/UT']}
                      />
                    </g>
                  </g>
                </>
              );
            })}
          </>
        </g>
      </svg>
    </>
  );
};

export default BarChart;
